//----------------------------------------------------------------------
// This algorithm calculates regressions polynomes of various
// dimensions, coefficients and any combination of potences.
// Released 22. September 2010 by Serge Zihlmann
// THIS CODE IS FREE TO BE USED IN ANY KIND OF SOFTWARE
// For further questions contact: powerfit( at@at )thirdway.ch
//----------------------------------------------------------------------
//--------------------------------------------------------
function [y] = PowerFit(data, zval, polynome)
  
  Error = 0;                //Start without errors
  [ndata dimD] = size(data); //Get amount of datapoints
  
  //get number of free variables of polynome
  [dim nCoeff] = size(polynome); //Number of dimensions and coefficients
  
  //Check for Errors
  if (dim <> dimD)
    printf("Different dimensions of polynome and data\n"); 
    Error = 1
  end
   if (ndata <> size(zval))
     printf("Different amount of data\n"); 
    Error = 1
  end 
  
  
  if (Error <> 1) //only start if no errors occur 
  //generate exponent tables
  ExpMat = zeros(nCoeff,nCoeff,dim);
  for i=1:nCoeff
    for k=1:dim
      ExpMat(i,:,k) = polynome(k,:)+polynome(k,i);
    end
  end
  ExpVec = polynome'; 
  
  // Generate matrices to fill
  A = zeros(nCoeff,nCoeff); // Final matrix
  b = zeros(nCoeff,1);      // Final solution vector

  // Build up matrix A
  for i=1:nCoeff
    for k=1:nCoeff
      t = ones(ndata,1);        // Temporary vector
      for r=1:dim
        t = t .* (data(:,r).^ExpMat(i,k,r));
      end
      A(i,k) = sum(t);
    end
  end
  
  // Build up vector b
  for i=1:nCoeff
      t = zval;
      for k=1:dim
        t = t .* (data(:,k).^ExpVec(i,k));
      end
      b(i) = sum(t);
  end
  
  // Solve the system A*y=b for y
  y = inv(A)*b; //calculate solution, return parameters
  end // end error
endfunction
//--------------------------------------------------------
//Evaluates polynome with coefficients, x=datapoint
function [y]=EvalPol(polynome, x, coefficients)
  [cx cy] = size(polynome);  
  y = coefficients';
  for i=1:cx
    y = y.*(x(i).^polynome(i,:));
  end
  y = sum(y);
endfunction
//--------------------------------------------------------
//--Prints the input polynome in a pretty shape 
function [y]=PrettyPrintF(polynom, coeffName, dimName)
  //prints out the symbolic function defined in polynom
  [cx cy] = size(polynom)
  printf("Regression function:\n");
  printf("--------------------\nf(");
  
  [ax ay] = size(dimName);
  [bx by] = size(coeffName);
    
  for i=1:cx
    if (i<=ay)
      printf("%s", dimName(i));
    else
      printf("D%i",i);
    end
    if(i<>cx) printf(","); end
  end
  printf(") = ");
  U = [];
  for i=1:cy
    if (i<=by)
      U = U + coeffName(i);
    else
      U = U + "a"+string(i); 
    end
    
    for k=1:cx
      if(polynom(k,i)<>0)//if power of x is greater than zero
        if (k<=ay)
          U = U + dimName(k);
        else
          U = U + "D" + string(k);
        end
        
        if((polynom(k,i)<>0) & (polynom(k,i)<>1))
          U = U+"^"+string(polynom(k,i));  
        end
      end     
    end

    if(i<cy)
      U = U + " + ";
    end
  end
  printf("%s",U);
  printf("\n\n");
  y = []; //dont return anything
endfunction